from flask import Blueprint, request
from flask_jwt_extended import jwt_required, get_jwt_identity
from flask_restful import Api, Resource

from portal.rest import submission_schema, submission_state_schema, review_schema, reviews_schema
from portal.tools.decorators import error_handler
from portal.tools.logging import log
from portal.service import service
from portal.service.errors import PortalAPIError

submissions = Blueprint('submissions', __name__, url_prefix='/submissions')
submissions_api = Api(submissions)


class SubmissionResource(Resource):
    @error_handler
    def get(self, sid):
        log.info(f"GET to {request.url}")
        submission = service.find_submission(sid)
        return submission_schema.dump(submission)

    @error_handler
    def delete(self, sid):
        log.info(f"GET to {request.url}")
        submission = service.find_submission(sid)
        service.delete_entity(submission)
        return '', 204


class SubmissionState(Resource):
    @error_handler
    def get(self, sid):
        log.info(f"GET to {request.url}")
        submission = service.find_submission(sid)
        # returning the enum value as a string should also be okay
        return submission_state_schema.dump(submission)[0], 200

    @error_handler
    def put(self, sid):
        log.info(f"PUT to {request.url}")
        submission = service.find_submission(sid)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for submission state update.')

        data = submission_state_schema.load(json_data)[0]

        submission.change_state(data['state'])
        service.write_entity(submission)
        log.info(f"Updated state of submission {sid} to {submission.state}")
        return '', 204


class SubmissionResult(Resource):
    @error_handler
    def put(self, sid):
        log.info(f"PUT to {request.url}")
        submission = service.find_submission(sid)

        # TODO load zip, save to storage


class SubmissionFiles(Resource):
    @error_handler
    def get(self, sid):
        # the user wants a zip or a json

        pass


class SubmissionSourceFiles(Resource):
    @error_handler
    def get(self, sid):
        pass


class SubmissionResultFiles(Resource):
    @error_handler
    def get(self, sid):
        pass


class SubmissionTestFiles(Resource):
    @error_handler
    def get(self, sid):
        pass


class SubmissionSources(Resource):
    @error_handler
    def get(self, sid):
        pass


class SubmissionResubmit(Resource):
    @error_handler
    def post(self, sid):
        log.info(f"POST to {request.url}")
        source_submission = service.find_submission(sid)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for submission resubmit.')

        data = submission_schema.load(json_data)[0]
        # create new submission by copying files from the source submission in storage
        new_submission = service.copy_submission(source_submission, note=data['note'])
        return submission_schema.dump(new_submission), 201


class SubmissionReview(Resource):
    @error_handler
    def get(self, sid):
        log.info(f"GET to {request.url}")
        submission = service.find_submission(sid)
        return review_schema.dump(submission.review)[0], 200

    @error_handler
    @jwt_required
    def post(self, sid):
        log.info(f"GET to {request.url}")
        submission = service.find_submission(sid)
        user = service.find_user(get_jwt_identity())

        if not user:
            raise PortalAPIError(401, message="Invalid access token: user not found.")

        if not submission.review:
            service.create_review(submission)
        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for group creation.')

        data = review_schema.load(json_data)[0]

        # also writes to db
        service.create_review_items(review=submission.review, items=data['review_items'], author=user)
        log.info(f"Added review items {data['review_items']} to review {submission.review.id} "
                 f"for submission {submission.id}")
        return review_schema.dump(submission.review)[0], 201


submissions_api.add_resource(SubmissionResource, '/<string:sid>')
submissions_api.add_resource(SubmissionState, '/<string:sid>/state')
submissions_api.add_resource(SubmissionResult, '/<string:sid>/result')
submissions_api.add_resource(SubmissionResubmit, '/<string:sid>/resubmit')

submissions_api.add_resource(SubmissionFiles, '/<string:sid>/files')
submissions_api.add_resource(SubmissionSourceFiles, '/<string:sid>/files/sources')
submissions_api.add_resource(SubmissionResultFiles, '/<string:sid>/files/results')

submissions_api.add_resource(SubmissionTestFiles, '/<string:sid>/test_files')
submissions_api.add_resource(SubmissionSources, '/<string:sid>/sources')

submissions_api.add_resource(SubmissionReview, '/<string:sid>/review')
